@@ -24,6 +24,7 @@
 
 #include "xo_config.h"
 #include "xo.h"
+#include "xo_private.h"
 #include "xo_encoder.h"
 
 #ifdef HAVE_DLFCN_H
@@ -113,6 +114,7 @@ typedef struct xo_encoder_node_s {
     TAILQ_ENTRY(xo_encoder_node_s) xe_link; /* Next session */
     char *xe_name;			/* Name for this encoder */
     xo_encoder_func_t xe_handler;	/* Callback function */
+    xo_whiteboard_func_t xe_wb_marker;	/* Whiteboard marker function */
     void *xe_dlhandle;			/* dlopen handle */
 } xo_encoder_node_t;
 
@@ -206,6 +208,33 @@ xo_encoder_find (const char *name)
     return NULL;
 }
 
+/*
+ * Return the encoder function for a specific shared library.  This is
+ * really just a means of keeping the annoying gcc verbiage out of the
+ * main code.  And that's only need because gcc breaks dlfunc's
+ * promise that I can cast it's return value to a function: "The
+ * precise return type of dlfunc() is unspecified; applications must
+ * cast it to an appropriate function pointer type."
+ */
+static xo_encoder_init_func_t
+xo_encoder_func (void *dlp)
+{
+    xo_encoder_init_func_t func;
+
+#if defined(HAVE_GCC) && __GNUC__ > 8
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif /* HAVE_GCC */
+
+    func = (xo_encoder_init_func_t) dlfunc(dlp, XO_ENCODER_INIT_NAME);
+
+#if defined(HAVE_GCC) && __GNUC__ > 8
+#pragma GCC diagnostic pop	/* Restore previous setting */
+#endif /* HAVE_GCC */
+
+    return func;
+}
+
 static xo_encoder_node_t *
 xo_encoder_discover (const char *name)
 {
@@ -234,7 +263,7 @@ xo_encoder_discover (const char *name)
 	 */
 	xo_encoder_init_func_t func;
 
-	func = (xo_encoder_init_func_t) dlfunc(dlp, XO_ENCODER_INIT_NAME);
+	func = xo_encoder_func(dlp);
 	if (func) {
 	    xo_encoder_init_args_t xei;
 
@@ -246,6 +275,7 @@ xo_encoder_discover (const char *name)
 		xep = xo_encoder_list_add(name);
 		if (xep) {
 		    xep->xe_handler = xei.xei_handler;
+		    xep->xe_wb_marker = xei.xei_wb_marker;
 		    xep->xe_dlhandle = dlp;
 		}
 	    }
@@ -290,8 +320,21 @@ xo_encoder_init (xo_handle_t *xop, const char *name)
 {
     xo_encoder_setup();
 
-    const char *opts = strchr(name, ':');
+    char opts_char = '\0';
+    const char *col_opts = strchr(name, ':');
+    const char *plus_opts = strchr(name, '+');
+
+    /*
+     * Find the option-separating character (plus or colon) which
+     * appears first in the options string.
+     */
+    const char *opts = (col_opts == NULL) ? plus_opts
+	: (plus_opts == NULL) ? col_opts
+	: (plus_opts < col_opts) ? plus_opts : col_opts;
+
     if (opts) {
+	opts_char = *opts;
+
 	/* Make a writable copy of the name */
 	size_t len = strlen(name);
 	char *copy = alloca(len + 1);
@@ -325,11 +368,15 @@ xo_encoder_init (xo_handle_t *xop, const char *name)
 	}
     }
 
-    xo_set_encoder(xop, xep->xe_handler);
+    xo_set_encoder(xop, xep->xe_handler, xep->xe_wb_marker);
 
-    int rc = xo_encoder_handle(xop, XO_OP_CREATE, name, NULL, 0);
+    int rc = xo_encoder_handle(xop, XO_OP_CREATE, NULL, name, NULL, 0);
     if (rc == 0 && opts != NULL) {
-	rc = xo_encoder_handle(xop, XO_OP_OPTIONS, name, opts, 0);
+	xo_encoder_op_t op;
+
+	/* Encoder API is limited, so we're stuck with two different options */
+	op = (opts_char == '+') ? XO_OP_OPTIONS_PLUS : XO_OP_OPTIONS;
+	rc = xo_encoder_handle(xop, op, NULL, name, opts, 0);
     }
 
     return rc;
@@ -356,19 +403,6 @@ xo_encoder_create (const char *name, xo_xof_flags_t flags)
     return xop;
 }
 
-int
-xo_encoder_handle (xo_handle_t *xop, xo_encoder_op_t op,
-		   const char *name, const char *value, xo_xff_flags_t flags)
-{
-    void *private = xo_get_private(xop);
-    xo_encoder_func_t func = xo_get_encoder(xop);
-
-    if (func == NULL)
-	return -1;
-
-    return func(xop, op, name, value, private, flags);
-}
-
 const char *
 xo_encoder_op_name (xo_encoder_op_t op)
 {
@@ -391,6 +425,26 @@ xo_encoder_op_name (xo_encoder_op_t op)
 	/* 15 */ "attr",
 	/* 16 */ "version",
 	/* 17 */ "options",
+	/* 18 */ "options-plus",
+	/* 19 */ "deadend",
+    };
+
+    if (op > sizeof(names) / sizeof(names[0]))
+	return "unknown";
+
+    return names[op];
+}
+
+const char *
+xo_whiteboard_op_name (xo_whiteboard_op_t op)
+{
+    static const char *names[] = {
+	/*  0 */ "unknown",
+	/*  1 */ "init",
+	/*  2 */ "record",
+	/*  3 */ "allow",
+	/*  4 */ "deny",
+	/*  5 */ "clean",
     };
 
     if (op > sizeof(names) / sizeof(names[0]))
