@@ -21,7 +21,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -38,13 +38,11 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/syslog.h>
 #include <sys/uio.h>
 #include <sys/un.h>
-#include <netdb.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <paths.h>
@@ -53,14 +51,20 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <limits.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/sysctl.h>
 
 #include "xo_config.h"
+
+#ifdef HAVE_SYSCTLBYNAME
+#include <sys/sysctl.h>
+#endif /* HAVE_SYSCTLBYNAME */
+
 #include "xo.h"
+#include "xo_private.h"
 #include "xo_encoder.h"		/* For xo_realloc */
 #include "xo_buf.h"
 
@@ -88,21 +92,19 @@
 
 #if defined(__FreeBSD__)
 #define XO_DEFAULT_EID	2238
-#elif defined(__macosx__)
+#elif defined(__APPLE__)
 #define XO_DEFAULT_EID	63
 #else
 #define XO_DEFAULT_EID	32473	/* Fallback to the "example" number */
 #endif
 
-#ifdef _SC_HOST_NAME_MAX
-#define HOST_NAME_MAX _SC_HOST_NAME_MAX
+#ifndef XO_HOST_NAME_MAX
+#ifdef _POSIX_HOST_NAME_MAX
+#define XO_HOST_NAME_MAX _POSIX_HOST_NAME_MAX
 #else
-#define HOST_NAME_MAX 255
-#endif /* _SC_HOST_NAME_MAX */
-
-#ifndef UNUSED
-#define UNUSED __attribute__ ((__unused__))
-#endif /* UNUSED */
+#define XO_HOST_NAME_MAX 255
+#endif /* _POSIX_HOST_NAME_MAX */
+#endif /* XO_HOST_NAME_MAX */
 
 static int xo_logfile = -1;		/* fd for log */
 static int xo_status;			/* connection xo_status */
@@ -173,8 +175,8 @@ xo_send_syslog (char *full_msg, char *v0_hdr,
     int fd;
     int full_len = strlen(full_msg);
 
-    /* Output to stderr if requested. */
-    if (xo_logstat & LOG_PERROR) {
+    /* Output to stderr if requested, then we've been passed a v0 header */
+    if (v0_hdr) {
         struct iovec iov[3];
         struct iovec *v = iov;
         char newline[] = "\n";
@@ -425,12 +427,13 @@ xo_set_syslog_handler (xo_syslog_open_t open_func,
     xo_syslog_close = close_func;
 }
 
-static size_t
-xo_snprintf (char *out, size_t outsize, const char *fmt, ...)
+static ssize_t
+xo_snprintf (char *out, ssize_t outsize, const char *fmt, ...)
 {
-    int status;
-    size_t retval = 0;
+    ssize_t status;
+    ssize_t retval = 0;
     va_list ap;
+
     if (out && outsize) {
         va_start(ap, fmt);
         status = vsnprintf(out, outsize, fmt, ap);
@@ -443,10 +446,11 @@ xo_snprintf (char *out, size_t outsize, const char *fmt, ...)
         }
         va_end(ap);
     }
+
     return retval;
 }
 
-static int
+static xo_ssize_t
 xo_syslog_handle_write (void *opaque, const char *data)
 {
     xo_buffer_t *xbp = opaque;
@@ -545,7 +549,7 @@ xo_vsyslog (int pri, const char *name, const char *fmt, va_list vap)
     } else
 	gettimeofday(&tv, NULL);
 
-    (void) localtime_r(&tv.tv_sec, &tm);
+    (void) localtime_r((time_t *) &tv.tv_sec, &tm);
 
     if (xo_logstat & LOG_PERROR) {
 	/*
@@ -579,12 +583,13 @@ xo_vsyslog (int pri, const char *name, const char *fmt, va_list vap)
      * Add HOSTNAME; we rely on gethostname and don't fluff with
      * ip addresses.  Might need to revisit.....
      */
-    char hostname[HOST_NAME_MAX];
+    char hostname[XO_HOST_NAME_MAX + 1];
     hostname[0] = '\0';
     if (xo_unit_test)
 	strcpy(hostname, "worker-host");
     else
-	(void) gethostname(hostname, sizeof(hostname));
+	(void) gethostname(hostname, sizeof(hostname) - 1);
+    hostname[XO_HOST_NAME_MAX] = '\0'; /* Ensure NUL-terminated */
 
     xb.xb_curp += xo_snprintf(xb.xb_curp, xo_buf_left(&xb), "%s ",
 			      hostname[0] ? hostname : "-");
