@@ -12,6 +12,7 @@
 #include <stdint.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/param.h>
 
 #include "xo.h"
 #include "xo_encoder.h"
@@ -50,6 +51,9 @@ main (int argc, char **argv)
 	{ "sold", "number", "Number of items sold" },
 	{ XO_INFO_NULL },
     };
+
+    char name[] = "test_01.test";  /* test trimming of xo_program */
+    argv[0] = name;
     
     argc = xo_parse_args(argc, argv);
     if (argc < 0)
@@ -70,6 +74,8 @@ main (int argc, char **argv)
 	    xo_set_flags(NULL, XOF_XPATH);
 	else if (xo_streq(argv[argc], "info"))
 	    xo_set_flags(NULL, XOF_INFO);
+	else if (xo_streq(argv[argc], "debug"))
+	    xo_set_flags(NULL, XOF_DEBUG);
         else if (xo_streq(argv[argc], "error")) {
             close(-1);
             xo_err(1, "error detected");
@@ -79,7 +85,7 @@ main (int argc, char **argv)
     xo_set_info(NULL, info, -1);
     xo_set_flags(NULL, XOF_KEYS);
 
-    xo_open_container_h(NULL, "top");
+    xo_open_container_h(NULL, "top-level");
 
     xo_emit("static {:type/ethernet} {:type/bridge} {:type/%4du} {:type/%3d}",
 	    18, 24);
@@ -101,10 +107,19 @@ main (int argc, char **argv)
 
     xo_emit("{La:} {a:}\n", "Label text", "label", "value");
 
+    const char *title = "My Title";
+    xo_emit_field("T", title, "%s\n", NULL, NULL);
+
     xo_emit_field("Vt", "max-chaos", NULL, NULL, "  very  ");
     xo_emit_field("V", "min-chaos", "%d", NULL, 42);
     xo_emit_field("V", "some-chaos", "%d\n", "[%d]", 42);
 
+    xo_attr("test-attr", "attr-value");
+    xo_emit_field_h(NULL, ",leaf-list,quotes", "sku", "%s-%u", "%s-000-%u",
+		    "gum", 1412);
+    xo_emit_field_h(NULL, ",leaf-list,quotes", "sku", "%s-%u", "%s-000-%u",
+		    "sum", 4121);
+
     xo_emit("Connecting to {:host}.{:domain}...\n", "my-box", "example.com");
 
     xo_attr("test", "value");
@@ -245,7 +260,15 @@ main (int argc, char **argv)
 	    "/some/file", (int) 0640, 8, 1,
 	    10, "user", 12, "group");
 
-    xo_close_container_h(NULL, "top");
+    /* Test retain flag for dynamic data */
+    xo_set_flags(NULL, XOF_RETAIN_ALL);
+    char buf[] = "Testing...{:one/%d}...{:two/%d}...{:three/%d}\n";
+    xo_emit(buf, 1, 2, 3);
+    xo_emit(buf, 1, 2, 3);
+    buf[0] = 'X';
+    xo_emit(buf, 1, 2, 3);
+
+    xo_close_container_h(NULL, "top-level");
 
     xo_finish();
 
