@@ -12,15 +12,16 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "xo.h"
 #include "xo_encoder.h"
 
-#include "xo_humanize.h"
-
 int
 main (int argc, char **argv)
 {
+    xo_set_program("test_02");
+
     argc = xo_parse_args(argc, argv);
     if (argc < 0)
 	return 1;
@@ -49,10 +50,10 @@ main (int argc, char **argv)
 
     xo_open_container("data");
 
-    xo_emit("{kt:name/%-*.*s}{eq:flags/0x%x}",
+    xo_emit("{ket:name/%-*.*s}{eq:flags/0x%x}",
 	    5, 5, "em0", 34883);
 
-    xo_emit("{d:/%-*.*s}{etk:name}{eq:flags/0x%x}",
+    xo_emit("{d:/%-*.*s}{tek:name}{eq:flags/0x%x}",
 	    5, 5, "em0", "em0", 34883);
 
     xo_emit("We are {{emit}}{{ting}} some {:what}\n", "braces");
@@ -83,6 +84,8 @@ main (int argc, char **argv)
     for (i = 0; i < 5; i++)
 	xo_emit("{lw:bytes/%d}{Np:byte,bytes}\n", i);
 
+    xo_emit("{Lc:Low\\/warn granularity}{P:\t}{:granularity-lw/%d}{Uw:/%sh}\n",
+	    155, "mA");
 
     xo_emit("{:mbuf-current/%u}/{:mbuf-cache/%u}/{:mbuf-total/%u} "
 	    "{N:mbufs <&> in use (current\\/cache\\/total)}\n",
@@ -137,6 +140,20 @@ main (int argc, char **argv)
     xo_emit("{W:/%s more warning%s}\n", "two", "s" );
     xo_emit("{L:/V1\\/V2 packet%s}: {:count/%u}\n", "s", 10);
 
+    long long long_long = -45;
+    unsigned long long ulong_long = 45;
+
+    xo_open_container("long-long-test");
+    xo_emit("a {:lsize/%u} long: {:lval/%lld}\n"
+	    "a {:usize/%u} ulong: {:uval/%llu}\n"
+	    "a {:lsize/%u} long: {:lval/%lld}\n"
+	    "a {:usize/%u} ulong: {:uval/%llu}\n",
+	    sizeof(long_long), long_long, sizeof(ulong_long), ulong_long,
+	    sizeof(long_long), long_long, sizeof(ulong_long), ulong_long);
+    xo_close_container("long-long-test");
+
+    xo_emit("{:value/%16lld}\n", 123412341234LL);
+
     int test = 4;
     xo_emit("{:test/%04d} {L:/tr%s}\n", test, (test == 1) ? "y" : "ies");
 
@@ -144,6 +161,10 @@ main (int argc, char **argv)
 	       "ten yard penalty", "first down");
 
     xo_error("Shut 'er down, Clancey!  She's a-pumpin' mud!  <>!,\"!<>\n");
+    xo_error("err message (%d)", 1);
+    xo_error("err message (%d)\n", 2);
+    xo_errorn("err message (%d)", 1);
+    xo_errorn("err message (%d)\n", 2);
 
     xo_close_container("data");
 
